window.Gleam.initSearch([{"doc":"validated","title":"validated","content":"# validated\n\nEasily accumulate errors in Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/validated)](https://hex.pm/packages/validated)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/validated/)\n\n```gleam\nimport gleam/string\nimport validated.{type Validated, Valid}\nimport validated as v\n\npub opaque type Form {\n  Form(email: String, age: Int)\n}\n\npub fn valid_form(email: String, age: Int) -&gt; Result(Form, List(String)) {\n  do_valid_form(email, age) |&gt; v.to_result\n}\n\nfn do_valid_form(email: String, age: Int) -&gt; Validated(Form, String) {\n  use email, _ &lt;- v.do(validate_email(email))\n  use age, _ &lt;- v.do(validate_age(age))\n  Valid(Form(email:, age:))\n}\n\nfn validate_email(email: String) -&gt; Validated(String, String) {\n  case string.contains(email, &quot;@&quot;) {\n    True -&gt; Ok(email)\n    False -&gt; Error(&quot;email addresses must include &#39;@&#39;&quot;)\n  }\n  |&gt; v.string\n}\n\nfn validate_age(age: Int) -&gt; Validated(Int, String) {\n  case age &gt;= 18 {\n    True -&gt; Ok(age)\n    False -&gt; Error(&quot;you must be 18 or older&quot;)\n  }\n  |&gt; v.int\n}\n\npub fn main() {\n  let assert Ok(Form(&quot;lucy@example.com&quot;, 18)) =\n    valid_form(&quot;lucy@example.com&quot;, 18)\n\n  let assert Error([&quot;email addresses must include &#39;@&#39;&quot;]) =\n    valid_form(&quot;lucy&quot;, 18)\n\n  let assert Error([\n    &quot;email addresses must include &#39;@&#39;&quot;,\n    &quot;you must be 18 or older&quot;,\n  ]) = valid_form(&quot;lucy&quot;, 1)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/validated&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"validated","title":"Validated","content":"pub type Validated(a, e) {\n  Valid(a)\n  Invalid(a, List(e))\n}\n `Validated` represents the result of something that may succeed or not.\n It is similar to `Result`, except that it will accumulate multiple errors\n instead of stopping at the first error.\n `Valid` means it was successful, `Invalid` means it was not successful.\nValid(a)\n\n\nInvalid(a, List(e))\n Invalid requires a default value to enable validation in `use` expressions\n\nSynonyms:\nvalidated.Validated\nvalidated Validated","url":"validated.html#Validated"},{"doc":"validated","title":"Validator","content":"pub type Validator(in, out, error) =\n  fn(in) -&gt; Validated(out, error)\n A function that takes some input and returns a `Validated`\n\nSynonyms:\nvalidated.Validator\nvalidated Validator","url":"validated.html#Validator"},{"doc":"validated","title":"all","content":"pub fn all(\n  validateds: List(Validated(a, b)),\n) -&gt; Validated(List(a), b)\n Combines a list of `Validateds` into a single `Validated`.\n If all elements in the list are `Valid` then the function returns a `Valid`\n holding the list of values.\n Otherwise an `Invalid` is returned that combines all the errors.\n `Valid([])` is returned if the list is empty.\nSynonyms:\nvalidated.all\nvalidated all","url":"validated.html#all"},{"doc":"validated","title":"bit_array","content":"pub fn bit_array(\n  value: Result(BitArray, a),\n) -&gt; Validated(BitArray, a)\n Creates a `Validated` from a `Result`.\n It uses an empty `BitArray` as the default value in case of failure.\nSynonyms:\nvalidated.bit_array\nvalidated bit_array","url":"validated.html#bit_array"},{"doc":"validated","title":"bool","content":"pub fn bool(value: Result(Bool, a)) -&gt; Validated(Bool, a)\n Creates a `Validated` from a `Result`.\n It uses `False` as the default value in case of failure.\nSynonyms:\nvalidated.bool\nvalidated bool","url":"validated.html#bool"},{"doc":"validated","title":"dict","content":"pub fn dict(\n  value: Result(Dict(a, b), c),\n) -&gt; Validated(Dict(a, b), c)\n Creates a `Validated` from a `Result`.\n It uses an empty `Dict` as the default value in case of failure.\nSynonyms:\nvalidated.dict\nvalidated dict","url":"validated.html#dict"},{"doc":"validated","title":"do","content":"pub fn do(\n  validated: Validated(a, b),\n  next: fn(a, Bool) -&gt; Validated(c, b),\n) -&gt; Validated(c, b)\n &quot;Updates&quot; a `Valid` value by passing its value, and a `Bool` indicating whether or not it is valid,\n  to a function that yields a `Validated`,\n and returning the yielded `Validated`. (This may &quot;replace&quot; the `Valid` with an `Invalid`.)\n\n If the input is an `Invalid` rather than an `Valid`, the function is still called using the default\n value from the `Invalid`. If the function succeeds, the `Invalid`&#39;s default is replaced with the `Valid` value.\n If the function fails, the first `Invalid` errors are combined with the returned `Invalid` errors.\n\n The `Bool` return value can be used to decide to short circuit further validation. See the `do_if`\n function for this use case. If you want to run all the validations regardless, you can ignore this value.\n\n `do` can be used in `use` expressions to ergonomically validate fields of a record\n\n ## Examples\n\n \n use a, _ &lt;- do(Valid(1))\n use b, _ &lt;- do(Valid(2))\n Valid(#(a, b))\n // -&gt; Valid(#(1, 2))\n \n\n \n use a, _ &lt;- do(Invalid(0, [Nil]))\n use b, _ &lt;- do(Valid(2))\n use c, _ &lt;- do(Invalid(0, [Nil]))\n Valid(#(a, b, c))\n // -&gt; Invalid(#(0, 2, 0), [Nil, Nil])\n \n\n \n use a, is_valid &lt;- do(Invalid(0, [Nil]))\n io.debug(is_valid)\n // -&gt; False\n use b, is_valid &lt;- do(Valid(2))\n io.debug(is_valid)\n // -&gt; True\n use c, is_valid &lt;- do(Invalid(0, [Nil]))\n io.debug(is_valid)\n // -&gt; False\n Valid(#(a, b, c))\n // -&gt; Invalid(#(0, 2, 0), [Nil, Nil])\n \nSynonyms:\nvalidated.do\nvalidated do","url":"validated.html#do"},{"doc":"validated","title":"do_if","content":"pub fn do_if(\n  when requirements: Bool,\n  do validated: Validated(a, b),\n  otherwise default: a,\n  next next: fn(a, Bool) -&gt; Validated(c, b),\n) -&gt; Validated(c, b)\n Like `do`, but only checks the input `Validated` if the requirements are `True`,\n otherwise it will return the provided default value as an `Invalid` with no errors.\n\n This function works well in `use` expressions with `do`.\n\n ## Examples\n\n \n use a, is_valid &lt;- do(Invalid(0, [Nil]))\n use a, _ &lt;- do_if(is_valid, Valid(1), a)\n use b, _ &lt;- do(Valid(2))\n Valid(#(a, b))\n // -&gt; Invalid(#(0, 2), [Nil])\n \n\n \n use username, is_valid &lt;- do(meets_requirements(username))\n use username, _ &lt;- do_if(is_valid, check_availability(username), username)\n use age, _ &lt;- do(Valid(30))\n Valid(User(username, age))\n \nSynonyms:\nvalidated.do_if\nvalidated do_if","url":"validated.html#do_if"},{"doc":"validated","title":"flatten","content":"pub fn flatten(\n  validated: Validated(Validated(a, b), b),\n) -&gt; Validated(a, b)\n Merges a nested `Validated` into a single layer.\nSynonyms:\nvalidated.flatten\nvalidated flatten","url":"validated.html#flatten"},{"doc":"validated","title":"float","content":"pub fn float(value: Result(Float, a)) -&gt; Validated(Float, a)\n Creates a `Validated` from a `Result`.\n It uses `0.0` as the default value in case of failure.\nSynonyms:\nvalidated.float\nvalidated float","url":"validated.html#float"},{"doc":"validated","title":"int","content":"pub fn int(value: Result(Int, a)) -&gt; Validated(Int, a)\n Creates a `Validated` from a `Result`.\n It uses `0` as the default value in case of failure.\nSynonyms:\nvalidated.int\nvalidated int","url":"validated.html#int"},{"doc":"validated","title":"is_invalid","content":"pub fn is_invalid(validated: Validated(a, b)) -&gt; Bool\n Checks whether the `Validated` is `Invalid`\nSynonyms:\nvalidated.is_invalid\nvalidated is_invalid","url":"validated.html#is_invalid"},{"doc":"validated","title":"is_valid","content":"pub fn is_valid(validated: Validated(a, b)) -&gt; Bool\n Checks whether the `Validated` is `Valid`\nSynonyms:\nvalidated.is_valid\nvalidated is_valid","url":"validated.html#is_valid"},{"doc":"validated","title":"list","content":"pub fn list(value: Result(List(a), b)) -&gt; Validated(List(a), b)\n Creates a `Validated` from a `Result`.\n It uses an empty list as the default value in case of failure.\nSynonyms:\nvalidated.list\nvalidated list","url":"validated.html#list"},{"doc":"validated","title":"map","content":"pub fn map(\n  validated: Validated(a, b),\n  f: fn(a) -&gt; c,\n) -&gt; Validated(c, b)\n Updates a value held within the `Valid` of a `Validated` by calling a given function on it.\n If the `Validated` is an `Invalid` rather than `Valid`, the function is called on the default value.\nSynonyms:\nvalidated.map\nvalidated map","url":"validated.html#map"},{"doc":"validated","title":"map_error","content":"pub fn map_error(\n  validated: Validated(a, b),\n  f: fn(List(b)) -&gt; List(c),\n) -&gt; Validated(a, c)\n Updates a value held within the `Invalid` of a `Validated` by calling a given function on it.\n If the result is `Valid` rather than `Invalid` the function is not called and the result stays the same.\nSynonyms:\nvalidated.map_error\nvalidated map_error","url":"validated.html#map_error"},{"doc":"validated","title":"optional","content":"pub fn optional(\n  value: Result(Option(a), b),\n) -&gt; Validated(Option(a), b)\n Creates a `Validated` from a `Result`.\n It uses `None` as the default value in case of failure.\nSynonyms:\nvalidated.optional\nvalidated optional","url":"validated.html#optional"},{"doc":"validated","title":"replace","content":"pub fn replace(\n  validated: Validated(a, b),\n  value: c,\n) -&gt; Validated(c, b)\n Replace the value within a `Validated`\nSynonyms:\nvalidated.replace\nvalidated replace","url":"validated.html#replace"},{"doc":"validated","title":"result","content":"pub fn result(value: Result(a, b), default: a) -&gt; Validated(a, b)\n Creates a `Validated` from a `Result`.\n It requires a default value in case of failure.\nSynonyms:\nvalidated.result\nvalidated result","url":"validated.html#result"},{"doc":"validated","title":"run","content":"pub fn run(\n  validator: fn(a) -&gt; Validated(b, c),\n  input: a,\n) -&gt; Validated(b, c)\n Run a `Validator` function.\nSynonyms:\nvalidated.run\nvalidated run","url":"validated.html#run"},{"doc":"validated","title":"run_all","content":"pub fn run_all(\n  validators: List(fn(a) -&gt; Validated(b, c)),\n  input: a,\n) -&gt; Validated(Nil, c)\n Run all the `Validators` in order on the given input.\n It will accumulate all the errors from all of the `Validators`.\n\n If there are no errors, or if the list is empty, `Valid(Nil)` is returned.\nSynonyms:\nvalidated.run_all\nvalidated run_all","url":"validated.html#run_all"},{"doc":"validated","title":"string","content":"pub fn string(value: Result(String, a)) -&gt; Validated(String, a)\n Creates a `Validated` from a `Result`.\n It uses an empty string as the default value in case of failure.\nSynonyms:\nvalidated.string\nvalidated string","url":"validated.html#string"},{"doc":"validated","title":"to_result","content":"pub fn to_result(\n  validated: Validated(a, b),\n) -&gt; Result(a, List(b))\n Convert a `Validated` into a `Result`\nSynonyms:\nvalidated.to_result\nvalidated to_result","url":"validated.html#to_result"},{"doc":"validated","title":"try_map","content":"pub fn try_map(\n  validated: Validated(a, b),\n  default: c,\n  f: fn(a) -&gt; Result(c, b),\n) -&gt; Validated(c, b)\n &quot;Updates&quot; a `Valid` value by passing its value to a function that yields a `Result`,\n converting that `Result` to a `Validated`, and returning the yielded `Validated`. (This may &quot;replace&quot;\n the `Valid` with an `Invalid`.)\n\n If the input is an `Invalid` rather than an `Valid`, the function is still called using the default\n value from the `Invalid`. If the function succeeds, the `Invalid`&#39;s default is replaced with the `Ok` value.\n If the function fails, the first `Invalid` errors are combined with the returned `Error`, and the default value\n is replaced with the provided default.\nSynonyms:\nvalidated.try_map\nvalidated try_map","url":"validated.html#try_map"},{"doc":"validated","title":"unwrap","content":"pub fn unwrap(validated: Validated(a, b)) -&gt; a\n Return the `Valid` value of the validated, or the default value if it is `Invalid`.\nSynonyms:\nvalidated.unwrap\nvalidated unwrap","url":"validated.html#unwrap"},{"doc":"validated","title":"validated","content":" This module provides the `Validated` type and associated functions\n to accumulate errors in an ergonomic way.\n\n ## Example\n ```gleam\n fn validate_form(email: String, age: Int) -&gt; Validated(Form, String) {\n   use email, is_valid &lt;- v.do(validate_email(email))\n   use email , _ &lt;- v.do_if(is_valid, check_database(email), email)\n   use age, _ &lt;- v.do(validate_age(age))\n   Valid(Form(email:, age:))\n }\n\n validated_form(&quot;lucy@example.com&quot;, 20)\n // -&gt; Valid(Form(&quot;lucy@example.com&quot;, 20))\n\n validated_form(&quot;exists@example.com&quot;, 5)\n // -&gt; Invalid(Form(&quot;&quot;, 0), [&quot;email address already exists&quot;, &quot;must be 18 or older&quot;])\n\n validated_form(&quot;asdf&quot;, 5)\n // -&gt; Invalid(Form(&quot;&quot;, 0), [&quot;not a valid email&quot;, &quot;must be 18 or older&quot;])\n ```\n\n This API is possible because a `Validated` requires a &quot;default&quot; value\n in case of failure. That way, the default value is passed through to\n continue the validation.\n\n Since a value within a validation block may or may not be the default\n value, it is important to **never perform side-effects inside the\n validation**.","url":"validated.html"}]);